/* The PostsDAO to be constructed with a connected database object */
function PostsDAO(db) {
    "use strict";
    if (false === (this instanceof PostsDAO)) {
        console.log('PostsDAO constructor called without "new" operator');
        return new PostsDAO(db);
    }

    this.insertEntry = function (title, body, tags, author, callback) {
        "use strict";
        console.log("inserting blog entry" + title + body);

        // permalink to not include whitespace
        var permalink = title.replace( /\s/g, '_' );
        permalink = permalink.replace( /\W/g, '' );

        // Build new post
        var post = {"title": title,
                "author": author,
                "body": body,
                "permalink":permalink,
                "tags": tags,
                "comments": [],
		"type":"post",
                "date": new Date()}

        // insert the post
	db.save('POSTID_'+Math.random(), post, function (err, res) {
      	      if (err) {
		  // Handle error
		  res += ' SAVE ERROR: Could not save record!!\n';
		  callback(err, null);	
	      } else {
		  // Handle success
		  res += ' SUCESSFUL SAVE\n';
		  callback(err, permalink);
	      }
	   });


    }

    this.getPosts = function(num, callback) {
        "use strict";

	db.view('post/byPosts', { }, function (err, doc) {
      		console.dir(doc);
            "use strict";

            if (err) return callback(err, null);

            console.log("Found " + doc.length + " posts");

            callback(null, doc);

	});

    }

    this.getPostsByTag = function(tag, num, callback) {
        "use strict";
	var tagArray = new Array();
	tagArray[0] = tag;

	db.view('post/byTags', { key : tagArray }, function (err, doc) {
      		console.dir(doc);
            "use strict";

            if (err) return callback(err, null);

            console.log("Found " + doc.length + " posts");

            callback(null, doc);

	});


    }

    this.getPostByPermalink = function(permalink, callback) {
        "use strict";
	db.view('post/byPermalinks', { key : permalink }, function (err, doc) {
      	    console.dir("in permalink --" + doc);
            "use strict";

            if (err) return callback(err, null);

            console.log("Found " + doc.length + " posts");

            callback(null, doc);

	});

    }

}

module.exports.PostsDAO = PostsDAO;
